/*
    NaturalOWL. 
    Copyright (C) 2008  Dimitrios Galanis and Giorgos Karakatsiotis.
    Natural Language Processing Group, Department of Informatics, 
    Athens University of Economics and Business, Greece.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package gr.aueb.cs.nlg.NLFiles;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.util.iterator.*;
import com.hp.hpl.jena.datatypes.*;

import java.io.*;


import gr.aueb.cs.nlg.*;

public class UserModellingManager extends NLGFileManager 
{
        
    
    // resources
    public static final String UserModellingRes = "UserModelling";
    public static final String UserTypeRes = "UserType";
    public static final String PropertyRes = "Property";
    public static final String MicroplanRes = "Microplan";
        
    // properties
    public static final String UserTypesPrp = "UserTypes";
        
    public static final String AppropriatenessPrp= "Appropriateness";
    public static final String AppropPrp = "Approp";
    
    public static final String forUserTypePrp = "forUserType";
    public static final String AppropValuePrp = "AppropValue";    
    
    public static final String forInstancePrp = "forInstance";
    public static final String forOwlClassPrp = "forOwlClass";
    
    public static final String InterestValuePrp = "InterestValue";
    public static final String RepetitionsValuePrp = "Repetitions";
    
    
    public static final String PropertiesInterestsRepetitionsPrp = "PropertiesInterestsRepetitions";
    
    public static final String DPInterestRepetitionsPrp = "DPInterestRepetitions";
    public static final String CDPInterestRepetitionsPrp = "CDPInterestRepetitions";
    public static final String IPInterestRepetitionsPrp = "IPInterestRepetitions";
                    
    
    public static final String ClassInterestsRepetitionsPrp = "ClassInterestsRepetitions";   
    
    public static final String MaxFactsPerSentencePrp = "MaxFactsPerSentence";
    public static final String SynthesizerVoicePrp = "SynthesizerVoice";
    public static final String FactsPerPagePrp = "FactsPerPage";
    public static final String LangPrp = "Language";
    
    public static final String DInterestRepetitionsPrp = "DInterestRepetitions";
    public static final String IInterestRepetitionsPrp = "IInterestRepetitions";

    
    // properties
    public Property UserTypesProperty = model.createProperty(owlnlNS + this.UserTypesPrp);
    public Property AppropriatenessProperty = model.createProperty(owlnlNS + this.AppropriatenessPrp);
    public Property AppropProperty = model.createProperty(owlnlNS + this.AppropPrp);
    public Property forUserTypeProperty = model.createProperty(owlnlNS + this.forUserTypePrp);
    public Property AppropValueProperty = model.createProperty(owlnlNS + this.AppropValuePrp);
   
    public Property forInstanceProperty =  model.createProperty(owlnlNS + this.forInstancePrp);
        
    public Property  DPInterestRepetitionsProperty = model.createProperty(owlnlNS + this.DPInterestRepetitionsPrp);
    public Property  CDPInterestRepetitionsProperty = model.createProperty(owlnlNS + this.CDPInterestRepetitionsPrp);
    public Property  IPInterestRepetitionsProperty = model.createProperty(owlnlNS + this.IPInterestRepetitionsPrp);
    
    public Property  InterestValueProperty = model.createProperty(owlnlNS + this.InterestValuePrp);        
    public Property  RepetitionsValueProperty = model.createProperty(owlnlNS + this.RepetitionsValuePrp);
    
    public Property forOwlClassProperty = model.createProperty(owlnlNS + this.forOwlClassPrp);
    public Property PropertiesInterestsRepetitionsProperty = model.createProperty(owlnlNS + this.PropertiesInterestsRepetitionsPrp);
    
    
    public Property MaxFactsPerSentenceProperty = model.createProperty(owlnlNS + this.MaxFactsPerSentencePrp);
    public Property SynthesizerVoiceProperty = model.createProperty(owlnlNS + this.SynthesizerVoicePrp);
    public Property FactsPerPageProperty = model.createProperty(owlnlNS + this.FactsPerPagePrp);
    public Property LangProperty = model.createProperty(owlnlNS + this.LangPrp);
    
    public Property ClassInterestsRepetitionsProperty = model.createProperty(owlnlNS + this.ClassInterestsRepetitionsPrp);
    
    public Property DInterestRepetitionsProperty = model.createProperty(owlnlNS + this.DInterestRepetitionsPrp);
    public Property IInterestRepetitionsProperty = model.createProperty(owlnlNS + this.IInterestRepetitionsPrp);
    
    
    public Resource MicroplansAndOrderingResourceType = null;
    
   
         
    //----------------------------------------------------------------------------
    public UserModellingManager(String xb){
        super(xb);             
    }

    //----------------------------------------------------------------------------

    
    public static void main(String args[]){//main

    }
}